note
	description: "Internal node in a code tree."

class
	INTERNAL_NODE

inherit
	NODE

create
	make

feature {NONE} -- Initialization

	make (l, r: NODE)
			-- Create an internal nodes with children `l' and `r'.			
		require
			l_exists: l /= Void
			r_exists: r /= Void
		do
			left := l
			right := r
			probability := l.probability + r.probability
		ensure
			left_set: left = l
			right_set: right = r
		end

feature -- Access

	left: NODE
			-- Left child.

	right: NODE
			-- Right child.

feature -- Traversal

	fill_codes (codes: V_TABLE [CHARACTER, STRING]; prefix_: STRING)
			-- Add paths to leafs of this subtree to `codes', if the path from root to this node is `prefix_'.
		do
			left.fill_codes (codes, prefix_ + "0")
			right.fill_codes (codes, prefix_ + "1")
		end

	decode_from (encoding: STRING; start: INTEGER): TUPLE [char: CHARACTER; pos: INTEGER]
			-- Assuming an encoding of of one of the leafs in this subtree starts from `start' in `encoding',
			-- return the character at the leaf and the position where the encoding of the next character starts.	
		do
			if encoding [start] = '0' then
				Result := left.decode_from (encoding, start + 1)
			else
				check encoding [start] = '1' end
				REsult := right.decode_from (encoding, start + 1)
			end
		end

feature -- Output

	show (indent: STRING)
			-- Pretty-print this subtree.
		do
			print (indent + "(" + ((probability * 100).rounded).out + "%%)")
			io.new_line
			left.show (indent + "  ")
			right.show (indent + "  ")
		end

invariant
	children_exist: left /= Void and right /= Void

end
