note
	description: "Leaf node in the code tree."

class
	LEAF

inherit
	NODE

create
	make

feature {NONE} -- Initialization

	make (char: CHARACTER; p: REAL_64)
			-- Create a leaf node that stores `char', which appears in the text with probability `p'.
		require
			p_in_range: 0.0 <= p and p <= 1.0
		do
			character := char
			probability := p
		ensure
			character_set: character = char
		end

feature -- Access

	character: CHARACTER
		-- Character stored in the leaf.

feature -- Traversal

	fill_codes (codes: V_TABLE [CHARACTER, STRING]; prefix_: STRING)
			-- Add paths to leafs of this subtree to `codes', if the path from root to this node is `prefix_'.
		do
			codes [character] := prefix_
		end

	decode_from (encoding: STRING; start: INTEGER): TUPLE [char: CHARACTER; pos: INTEGER]
			-- Assuming an encoding of of one of the leafs in this subtree starts from `start' in `encoding',
			-- return the character at the leaf and the position where the encoding of the next character starts.	
		require else
			start_can_be_after: 1 <= start and start <= encoding.count + 1
		do
			Result := [character, start]
		end

feature -- Output

	show (indent: STRING)
			-- Pretty-print this subtree.
		do
			print (indent + "(" + character.out + ", " + (probability * 100).rounded.out + "%%)")
			io.new_line
		end

end
