note
	description: "Huffman encoder."

class
	ENCODER

feature -- Main operations

	encode (text: STRING)
			-- Encode `text' and store result in `encoding'.
		require
			text_non_empty: text /= Void and then not text.is_empty
		do
		ensure
			encoding_non_empty: encoding /= Void and not encoding.is_empty
			encoding_binary: across encoding as c all c.item = '0' or c.item = '1' end
			code_tree_exists: code_tree /= Void
		end

feature -- Access

	encoding: STRING
			-- Result of the last encoding:
			-- a string consisting of charachers '0' and '1'
			-- (which could later be converted into a bitstring).

	code_tree: NODE
			-- Code tree built for the last encoding.

feature {NONE} -- Implementation

	probabilities: V_TABLE [CHARACTER, REAL_64]
			-- For each character in the text, the probability of its appearance.

	codes: V_TABLE [CHARACTER, STRING]
			-- For each character in the text, its Huffman code.

feature {NONE} -- Output

	show_probablities
			-- Pretty-print `probabilities'.
		require
			probabilities_exists: probabilities /= Void
		do
			across
				probabilities as cur
			loop
				print (cur.key.out + " " + (cur.item * 100).rounded.out + "%%")
				io.new_line
			end
		end

	show_code_tree
			-- Pretty-print `code_tree'.
		require
			code_tree_exists: code_tree /= Void
		do
			code_tree.show ("")
		end

	show_codes
			-- Pretty-print `codes'.	
		require
			codes: codes /= Void
		do
			across
				codes as cur
			loop
				print (cur.key.out + " " + cur.item)
				io.new_line
			end
		end
		
end
